@@ -79,11 +79,41 @@ ScriptModule::ScriptModule(const char *name)
 	cmd = name;
 }
 
+void addFunction(const char *id, unsigned args, Script::Function handler)
+{
+        Script::Fun *rf = new Script::Fun;
+        rf->id = id;        
+	rf->args = args;
+	rf->fn = handler;
+        rf->next = ScriptInterp::ifun;
+        ScriptInterp::ifun = rf;
+}
+
+void addConditional(const char *id, Script::Cond handler)
+{
+	Script::Test *test = new Script::Test;
+
+	test->id = id;
+	test->handler = handler;
+	test->next = ScriptInterp::test;
+	ScriptInterp::test = test;
+}
+
+void addAttribute(const char *id, Script::Meta handler)
+{
+	Script::Attr *attr = new Script::Attr;
+
+	attr->id = id;
+	attr->meta = handler;
+	attr->next = ScriptInterp::attr;
+	ScriptInterp::attr = attr;
+}
+
 ScriptModule *ScriptModule::find(const char *name)
 {
 	ScriptModule *mod = first;
 	char keybuf[33];
-	int len = 0;
+	unsigned len = 0;
 
 	while(*name && *name != '.' && len < sizeof(keybuf) - 1)
 		keybuf[len++] = *(name++);
@@ -130,7 +160,14 @@ bool Script::use(const char *name)
 
 	if(*name != '.' && *name != '/')
 	{
+#ifdef	__WIN32__
+		char *spath = getenv("windir");
+		if(!spath)
+			spath = "C:\\WINDOWS";
+		snprintf(buffer, sizeof(buffer), "%s\\ccscript2\\%s.pkg", spath, name);
+#else
 		snprintf(buffer, sizeof(buffer), "%s/%s.pkg",  SCRIPT_MODULE_PATH,  name);
+#endif
 		name = buffer;
 	}
 
