@@ -54,6 +54,10 @@ namespace ost {
 using namespace std;
 #endif
 
+//#ifdef	WIN32
+//Slog slog;
+//#endif
+
 static int keyindex(const char *keyword)
 {
 	unsigned key = 0;
@@ -64,11 +68,9 @@ static int keyindex(const char *keyword)
 	return key % KEYWORD_INDEX_SIZE;
 }
 
-ScriptCommand::ScriptCommand(const char *cfgfile) :
-Keydata(cfgfile), Mutex()
+ScriptCommand::ScriptCommand() :
+MemPager(), Mutex()
 {
-	char *scrname, *ext;
-	char buffer[256];
 	ifstream cf;
 
 	static Script::Define interp[] = {
@@ -76,22 +78,26 @@ Keydata(cfgfile), Mutex()
 		{"slog", &ScriptInterp::scrSlog, &ScriptCommand::chkHasArgs},
 		{"exit", &ScriptInterp::scrExit, &ScriptCommand::chkNoArgs},
 		{"set", &ScriptInterp::scrSet, &ScriptCommand::chkIgnore},
+		{"number", &ScriptInterp::scrNumber, &ScriptCommand::chkHasArgs},
 		{"clear", &ScriptInterp::scrClear, &ScriptCommand::chkHasVars},
 		{"try", &ScriptInterp::scrTry, &ScriptCommand::chkHasArgs},
 		{"goto", &ScriptInterp::scrGoto, &ScriptCommand::chkHasArgs},
 		{"call", &ScriptInterp::scrCall, &ScriptCommand::chkHasArgs},
 		{"gosub", &ScriptInterp::scrCall, &ScriptCommand::chkHasArgs},
+		{"source", &ScriptInterp::scrCall, &ScriptCommand::chkHasArgs},
 		{"pop", &ScriptInterp::scrPop, &ScriptCommand::chkIgnore},
 		{"return", &ScriptInterp::scrReturn, &ScriptCommand::chkIgnore},
 		{"select", &ScriptInterp::scrSelect, &ScriptCommand::chkHasArgs},
 		{"size", &ScriptInterp::scrSize, &ScriptCommand::chkHasArgs},
-		{"init", &ScriptInterp::scrInit, &ScriptCommand::chkIgnore},
+		{"init", &ScriptInterp::scrSet, &ScriptCommand::chkIgnore},
+		{"var", &ScriptInterp::scrVar, &ScriptCommand::chkHasArgs},
 		{"if", &ScriptInterp::scrIf, &ScriptCommand::chkHasArgs},
 		{"do", &ScriptInterp::scrDo, &ScriptCommand::chkIgnore},
 		{"for", &ScriptInterp::scrFor, &ScriptCommand::chkHasList},
 		{"read", &ScriptInterp::scrRead, &ScriptCommand::chkIgnore},
 		{"tryeach", &ScriptInterp::scrTryeach, &ScriptCommand::chkHasArgs},
 		{"foreach", &ScriptInterp::scrForeach, &ScriptCommand::chkHasArgs},
+		{"fordata", &ScriptInterp::scrFordata, &ScriptCommand::chkHasArgs},
 		{"repeat", &ScriptInterp::scrRepeat, &ScriptCommand::chkHasArgs},
 		{"dump", &ScriptInterp::scrDump, &ScriptCommand::chkHasArgs},
 		{"gather", &ScriptInterp::scrGather, &ScriptCommand::chkHasArgs},
@@ -105,12 +111,7 @@ Keydata(cfgfile), Mutex()
 		{"dec", &ScriptInterp::scrDec, &ScriptCommand::chkHasArgs},
 		{"once", &ScriptInterp::scrOnce, &ScriptCommand::chkHasArgs},
 		{"on", &ScriptInterp::scrOn, &ScriptCommand::chkHasArgs},
-		{"pack", &ScriptInterp::scrPack, &ScriptCommand::chkHasList},
-		{"unpack", &ScriptInterp::scrUnpack, &ScriptCommand::chkHasList},
-		{"packed", &ScriptInterp::scrPacked, &ScriptCommand::chkHasArgs},
 		{"lock", &ScriptInterp::scrLock, &ScriptCommand::chkIgnore},
-		{"has", &ScriptInterp::scrHas, &ScriptCommand::chkHasArgs},
-		{"missing", &ScriptInterp::scrMissing, &ScriptCommand::chkHasArgs},
 		{"const", &ScriptInterp::scrConst, &ScriptCommand::chkIgnore},
 		{"basename", &ScriptInterp::scrBasename, &ScriptCommand::chkHasArgs},
 		{"dirname", &ScriptInterp::scrDirname, &ScriptCommand::chkHasArgs},
@@ -141,9 +142,12 @@ Keydata(cfgfile), Mutex()
 		{"label", &ScriptInterp::scrLabel, &ScriptCommand::chkHasArgs},
 		{"ref", &ScriptInterp::scrRef, &ScriptCommand::chkHasArgs},
 		{"map", &ScriptInterp::scrMap, &ScriptCommand::chkHasArgs},
+		{"begin", &ScriptInterp::scrBegin, &ScriptCommand::chkIgnore},
+		{"end", &ScriptInterp::scrEnd, &ScriptCommand::chkNoArgs},
+		{"decimal", &ScriptInterp::scrDecimal, &ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
-	int i;
+	unsigned i;
 	memset(&keywords, 0, sizeof(keywords));
 	for(i = 0; i < TRAP_BITS; ++i)
 		traps[i] = "<undefined>";
@@ -157,61 +161,6 @@ Keydata(cfgfile), Mutex()
 	trap("exit");
 	trap("error");
 	load(interp);
-
-	// parse global definition entries
-
-	if(!isDir(SCRIPT_MODULE_PATH))
-		return;
-
-	if(!canAccess(SCRIPT_MODULE_PATH))
-		return;
-
-	Dir dir(SCRIPT_MODULE_PATH);
-	for(;;)
-	{
-		scrname = dir.getName();
-		if(!scrname)
-			return;
-
-		ext = strchr(scrname, '.');
-		if(!ext)
-			continue;
-		if(stricmp(ext, "cfg"))
-			continue;
-
-		snprintf(buffer, sizeof(buffer) - 1, "%s/%s",
-			SCRIPT_MODULE_PATH, scrname);
-
-		cf.open(buffer);
-		if(!cf.is_open())
-			continue;
-
-		for(;;)
-		{
-			cf.getline(buffer, sizeof(buffer) - 1);
-			if(cf.eof())
-				break;
-			scrname = buffer;
-			while(isspace(*scrname))
-				++scrname;
-
-			if(*scrname == '#' || !*scrname)
-				continue;
-
-			ext = scrname;
-
-			while(*ext && !isspace(*ext))
-				++ext;
-
-			*ext = 0;
-			while(isspace(*ext) || *ext == '=')
-				++ext;
-
-			setValue(scrname, ext);
-		}
-		cf.close();
-		cf.clear();
-	}
 }
 
 Script::Method ScriptCommand::getHandler(const char *keyword)
@@ -288,7 +237,7 @@ unsigned long ScriptCommand::getTrapMask(unsigned id)
 unsigned long ScriptCommand::getTrapMask(const char *trapname)
 {
 	unsigned long mask = 1;
-	int i;
+	unsigned i;
 
 	for(i = 0; i < TRAP_BITS; ++i)
 	{
@@ -350,7 +299,7 @@ char *ScriptCommand::chkUse(Line *line, ScriptImage *img)
 	}
 	return NULL;
 #else
-	return "dynamic loader unsupported"
+	return "dynamic loader unsupported";
 #endif
 }
 
@@ -360,7 +309,7 @@ char *ScriptCommand::chkModule(Line *line, ScriptImage *img)
 	char keybuf[32];
 
 	char *kw = line->cmd;
-	int len = 0;
+	unsigned len = 0;
 	while(*kw && len < sizeof(keybuf) - 1 && *kw != '.')
 		keybuf[len++] = *kw;
 	keybuf[len] = 0;
@@ -440,10 +389,24 @@ ScriptSymbol::ScriptSymbol(int size, int paging) :
 SharedMemPager(paging)
 {
 	symsize = size;
+	symlimit = paging - 32 - sizeof(Symbol);
 	memset(index, 0, sizeof(index));
 	trigger = NULL;
 }
 
+ScriptSymbol::~ScriptSymbol()
+{
+	Symbol *sym = index[SYMBOL_INDEX_SIZE];
+	Symbol *next;
+
+	while(sym)
+	{
+		next = sym->next;
+		delete[] sym;
+		sym = next;
+	}
+}
+
 void ScriptSymbol::setExclusive(bool enable)
 {
 	if(enable)
@@ -481,7 +444,7 @@ unsigned ScriptSymbol::gather(Symbol **idx, unsigned max, const char *prefix, co
 
 	enterMutex();
 
-	while(max && key < SYMBOL_INDEX_SIZE)
+	while(max && key <= SYMBOL_INDEX_SIZE)
 	{
 		node = index[key++];
 		while(node && max)
@@ -579,6 +542,7 @@ Script::Symbol *ScriptSymbol::getEntry(const char *symname, int size)
 	int key;
 	Symbol *sym;
 	ScriptSymbol *ref;
+	bool large = false;
 
 	enterMutex();
 
@@ -586,6 +550,7 @@ retry:
 	if(*symname == '%')
 		++symname;
 	key = getIndex(symname);
+large:
 	sym = index[key];
 	while(sym)
 	{
@@ -594,10 +559,17 @@ retry:
 
 		sym = sym->next;
 	}
+	if(!sym && key < SYMBOL_INDEX_SIZE)
+	{
+		key = SYMBOL_INDEX_SIZE;
+		goto large;
+	}
+
+	key = getIndex(symname);
 
 	if(sym)
 	{
-		if(sym->flags.type == ALIAS)
+		if(sym->flags.type == ALIAS || sym->flags.type == ARRAY)
 		{
 			symname = sym->data;
 			goto retry;
@@ -606,6 +578,7 @@ retry:
 		{
 			symname = sym->data + sizeof(ScriptSymbol *);
 			ref = *((ScriptSymbol **)sym->data);
+			leaveMutex();
 			return ref->getEntry(symname, size);
 		}
 	}
@@ -616,12 +589,21 @@ retry:
 		return sym;
 	}
 
-	sym = (Symbol *)alloc(sizeof(Symbol) + size);
+	if(size > symlimit)
+	{
+		key = SYMBOL_INDEX_SIZE;
+		large = true;
+		sym = (Symbol *)new char[sizeof(Symbol) + size];
+	}
+	else
+		sym = (Symbol *)alloc(sizeof(Symbol) + size);
+
 	sym->id = MemPager::alloc((char *)symname);
 	sym->next = index[key];
 	sym->flags.size = size;
 	sym->flags.commit = false;
 	sym->flags.initial = true;
+	sym->flags.large = large;
 	sym->flags.system = false;
 	sym->flags.readonly = false;
 	sym->flags.type = NORMAL;
@@ -651,7 +633,7 @@ Script::Symbol *ScriptSymbol::getAlias(const char *symname)
 
 	if(sym)
 	{
-		if(sym->flags.type != ALIAS)
+		if(sym->flags.type != ALIAS && sym->flags.type != ARRAY)
 			sym = NULL;
 	}
 	leaveMutex();
@@ -709,6 +691,62 @@ bool ScriptSymbol::setPointer(const char *id, void *data)
 	return true;
 }
 
+bool ScriptSymbol::swapSymbol(const char *oldname, const char *newname)
+{
+	unsigned ol = getIndex(oldname);
+	unsigned np = getIndex(newname);
+	Symbol *onode, *nnode;
+	Symbol *oprior = NULL, *nprior = NULL;
+	Symbol *onext, *nnext;
+
+	enterMutex();
+	onode = index[ol];
+	while(onode)
+	{
+		if(!stricmp(onode->id, oldname))
+			break;
+		oprior = onode;
+		onode = onode->next;
+	}
+
+	nnode = index[np];
+	while(nnode)
+	{
+		if(!stricmp(nnode->id, newname))
+			break;
+
+		nprior = nnode;
+		nnode = nnode->next;
+	}
+
+	if(!nnode || !onode)
+	{
+		leaveMutex();
+		return false;
+	}
+
+	nnext = nnode->next;
+	onext = onode->next;
+
+	if(nprior)
+		nprior->next = onode;
+	else
+		index[np] = onode;
+
+	if(oprior)
+		oprior->next = nnode;
+	else
+		index[ol] = nnode;
+
+	nnode->next = onext;
+	onode->next = nnext;
+	onode->id = newname;
+	nnode->id = oldname;
+
+	leaveMutex();
+	return true;
+}
+
 bool ScriptSymbol::setAlias(const char *id, const char *source)
 {
 	Symbol *sym;
@@ -732,6 +770,29 @@ bool ScriptSymbol::setAlias(const char *id, const char *source)
 	return true;
 }
 
+bool ScriptSymbol::setArray(const char *id, const char *source)
+{
+	Symbol *sym;
+
+	if(*id == '%')
+		++id;
+
+	if(*source == '%')
+		++source;
+
+	sym = getEntry(id, strlen(source));
+	if(!sym->flags.initial)
+		return false;
+
+	enterMutex();
+	strcpy(sym->data, source);
+	sym->flags.initial = false;
+	sym->flags.readonly = true;
+	sym->flags.type = ARRAY;
+	leaveMutex();
+	return true;
+}
+
 bool ScriptSymbol::makeCounter(const char *id)
 {
 	Symbol *sym;
@@ -895,9 +956,9 @@ bool ScriptSymbol::removeSymbol(Symbol *sym, const char *value)
 		++j;
 	}
 
-	if(sym->data[1] > i)
+	if((unsigned)sym->data[1] > i)
 		--sym->data[1];
-	if(sym->data[2] > i)
+	if((unsigned)sym->data[2] > i)
 		--sym->data[2];
 	leaveMutex();
 	return true;
@@ -919,7 +980,7 @@ bool ScriptSymbol::postSymbol(Symbol *sym, const char *value)
 
 	if(tail >= size && sym->flags.type == CACHE)
 	{
-		while(offset < size - 1)
+		while(offset < (unsigned)(size - 1))
 		{
 			strcpy(sym->data + 5 + offset * (rec + 1),
 				sym->data + 5 + (offset + 1) * (rec + 1));
@@ -953,7 +1014,7 @@ char *ScriptSymbol::readSymbol(Symbol *sym)
 {
 	long value;
 	char *data;
-	unsigned char head, rec, size;
+	unsigned char head = 0, rec, size;
 	switch(sym->flags.type)
 	{
 	case COUNTER:
@@ -1031,13 +1092,8 @@ char *ScriptSymbol::setConst(const char *id, const char *value)
 
 char *ScriptSymbol::setSymbol(const char *id, const char *value)
 {
-	char array[96];
-	int count;
-	int index;
-	char *cp;
 	long val;
 
-	Symbol *alias;
 	Symbol *sym = getEntry(id);
 	if(!sym)
 		return NULL;
@@ -1106,15 +1162,43 @@ char *ScriptSymbol::setSymbol(const char *id, int size)
 	return sym->data;
 }
 
-ScriptImage::ScriptImage(ScriptCommand *cmd)
+ScriptImage::ScriptImage(ScriptCommand *cmd, const char *symset) :
+Keydata(symset)
 {
+	static Script::Initial initial[] = {
+		{"script.error", 64, "none"},
+		{"script.trigger", 64, "none"},
+		{"script.home", 64, "none"},
+		{"script.token", 1, ","},
+		{"script.decimal", 1, "."},
+		{NULL, 0, NULL}
+	};
+
 	char buffer[256];
 	char *scrname, *ext;
 	cmds = cmd;
 	memset(index, 0, sizeof(index));
 	refcount = 0;
 	scrStream = (istream *)&scrSource;
+	ilist = NULL;
 
+	load(initial);
+
+#ifdef	__WIN32__
+	char *spath = getenv("windir");
+	if(!spath)
+		spath = "C:\\WINDOWS";
+
+	snprintf(buffer, sizeof(buffer), "%s\\ccscript2", spath);
+	if(!isDir(buffer))
+		return;
+
+	if(!canAccess(buffer))
+		return;
+
+	Dir dir(buffer);
+
+#else
 	if(!isDir(SCRIPT_MODULE_PATH))
 		return;
 
@@ -1122,6 +1206,7 @@ ScriptImage::ScriptImage(ScriptCommand *cmd)
 		return;
 
 	Dir dir(SCRIPT_MODULE_PATH);
+#endif
 	for(;;)
 	{
 		scrname = dir.getName();
@@ -1132,12 +1217,41 @@ ScriptImage::ScriptImage(ScriptCommand *cmd)
 			continue;
 		if(stricmp(ext, ".src"))
 			continue;
-		snprintf(buffer, sizeof(buffer) - 1, "%s/%s",
+#ifdef	__WIN32__
+		snprintf(buffer, sizeof(buffer), "%s\\ccscript2\%s",
+			spath, scrname); 
+#else
+		snprintf(buffer, sizeof(buffer), "%s/%s",
 			SCRIPT_MODULE_PATH, scrname);
+#endif
 		ScriptImage::compile(buffer);
 	}
 }
 
+void ScriptImage::initial(const char *keyword, const char *value, unsigned size)
+{
+	InitialList *init;
+
+	if(!size)
+		size = strlen(value);
+
+	init = (InitialList *)alloc(sizeof(InitialList));
+	init->name = alloc((char *)keyword);
+	init->size = size;
+	init->value = alloc((char *)value);
+	init->next = ilist;
+	ilist = init;
+}
+
+void ScriptImage::load(Initial *init)
+{
+	while(init->name)
+	{
+		initial(init->name, init->value, init->size);
+		++init;
+	}
+}
+
 void ScriptImage::purge(void)
 {
 	MemPager::purge();
@@ -1163,17 +1277,39 @@ char *ScriptImage::getToken(char **pre)
 	char *cp = temp + 1;
 	char *base = temp + 1;
 	char q;
+	int level;
 
 	if(pre)
 		if(*pre)
 		{
 			cp = *pre;
 			*pre = NULL;
+			return cp;
 		}
 
 	if(*bp == '=')
-	{
+	{	
 		++bp;
+       		if(*bp == '{')
+        	{
+                	level = -1;
+                	while(*bp)
+                	{
+                        	if(*bp == '}' && !level)
+                                	break;
+                        	if(*bp == '{')
+                                	++level;
+                        	else if(*bp == '}')
+                                	--level;
+                        	*(cp++) = *(bp++);
+                	}
+                	if(*bp == '}')
+                        	++bp;
+                	*cp = 0;
+			*base = 0x01;	// special token
+                	return base;
+        	}
+
 		if(*bp == ' ' || *bp == '\t' || !*bp)
 			return "";
 		if(*bp == '\"' || *bp == '\'')
@@ -1234,11 +1370,16 @@ char *ScriptImage::getToken(char **pre)
 		while(*bp == ' ' || *bp == '\t')
 			++bp;
 
-	if(!quote && *bp == '#')
+	if(!quote && *bp == '#' && (!bp[1] || bp[1] == '!' || isspace(bp[1])))
 		return NULL;
 
+	if(!quote && bp[0] == '%' && bp[1] == '%' && (!bp[2] || isspace(bp[2])))
+		return NULL;
+
+
 	if(!*bp)
 	{
+		paren = 0;
 		quote = false;
 		return NULL;
 	}
@@ -1249,12 +1390,41 @@ char *ScriptImage::getToken(char **pre)
 		quote = true;
 	}
 
+	if(!quote && *bp == '{')
+	{
+		level = -1;
+		while(*bp)
+		{
+			if(*bp == '}' && !level)
+				break;
+			if(*bp == '{')
+				++level;
+			else if(*bp == '}')
+				--level;
+			*(cp++) = *(bp++);
+		}
+		if(*bp == '}')
+			++bp;
+		*cp = 0;
+		return base;
+	}
+
 	if(!quote)
 	{
+		if(*bp == ',' && paren)
+		{
+			++bp;
+			return ",";
+		}			
 retry:
-		while(*bp && !isspace(*bp) && *bp != ',' && *bp != '=')
+		while(*bp && !isspace(*bp) && *bp != ',' && *bp != '=' && *bp != '(' && *bp != ')' )
 			*(cp++) = *(bp++);
 
+		if(*bp == '(')
+			++paren;
+		else if(*bp == ')')
+			--paren;
+
 		if(*bp == '=' && cp == base)
 		{
 			*(cp++) = *(bp++);
@@ -1267,8 +1437,11 @@ retry:
 			goto retry;
 		}
 
+		if((*bp == '(' || *bp == ')') && cp == base)
+			*(cp++) = *(bp++);
+
 		*cp = 0;
-		if(*bp == ',')
+		if(*bp == ',' && !paren)
 			++bp;
 		else if(*bp == '=')
 			*(--base) = *(bp);
@@ -1291,6 +1464,14 @@ retry:
 		return base;
 	}
 
+	if(*bp == '\\' && (bp[1] == '%' || bp[1] == '&' || bp[1] == '.'))
+	{
+		++bp;
+		*(cp++) = '{';
+		*(cp++) = *(bp++);
+	}
+
+requote:
 	if(isalnum(*bp) || strchr("~/:,. \t\'", *bp))
 	{
 		while(isalnum(*bp) || strchr("~=/:,. \t\'", *bp))
@@ -1299,9 +1480,37 @@ retry:
 	else while(!isspace(*bp) && *bp && *bp != '\"')
 		*(cp++) = *(bp++);
 
+	if(*bp == '\n' || !*bp)
+		paren = 0;
+
 	if(*bp == '\n' || !*bp || *bp == '\"')
 		quote = false;
 
+	if(*bp == '\\' && bp[1])
+	{
+		++bp;
+		switch(*bp)
+		{
+		case 0:
+			break;
+		case 't':
+			*(cp++) = '\t';
+			++bp;
+			break;
+		case 'b':
+			*(cp++) = '\b';
+			++bp;
+			break;
+		case 'n':
+			*(cp++) = '\n';
+			++bp;
+			break;
+		default:
+			*(cp++) = *(bp++);
+		}
+		goto requote;
+	}
+
 	if(*bp == '\n' || *bp == '\"')
 		++bp;
 
@@ -1343,7 +1552,7 @@ unsigned ScriptImage::gather(const char *suffix, Name **array, unsigned max)
 				++sort;
 			}
 			ins = count;
-			while(ins > sort)
+			while(ins > (int)sort)
 			{
 				array[ins] = array[ins - 1];
 				--ins;
@@ -1381,6 +1590,7 @@ Script::Name *ScriptImage::dupScript(const char *src, const char *dest)
 	scr->mode = Name::COPIED;
 	nscr->next = index[key];
 	index[key] = nscr;
+// COPY EVENTS???
 	duplock.leaveMutex();
 	return nscr;
 }
@@ -1449,28 +1659,35 @@ Script::Name *ScriptImage::include(const char *scrname)
 
 int ScriptImage::compile(const char *scrname, char *name)
 {
+	int rtn;
+
 	scrSource.open(scrname);
 	if(!scrSource.is_open())
 		return 0;
 
-	compile((istream *)&scrSource, name, scrname);
+	rtn = compile((istream *)&scrSource, name, scrname);
 	scrSource.close();
 	scrSource.clear();
+	return rtn;
 }
 
 int ScriptImage::compile(istream *str, char *name, const char *scrname)
 {
 	const char *basename = name;
 	char namebuf[256];
-	char *command, *token, *pretoken;
-	char *args[33];
+	char gvarname[128];
+	char *command, *token, *pretoken, *prefix;
+	char *args[SCRIPT_MAX_ARGS + 1];
+	int maxargs = SCRIPT_MAX_ARGS;
 	char *err, *cp;
-	bool trapflag, errflag, syncflag;
-	int argc, i, key, tlen;
-	unsigned short count, number, total = 0;
+	bool trapflag, errflag = false, syncflag = false;
+	int argc, key, tlen;
+	unsigned i;
+	unsigned short count, number, vsize, gvarlen, total = 0;
+	Name::Event *events, *esave;
 	Name *script;
 	Line *line, *last;
-	unsigned long addmask, submask, trapmask, mask, cmask;
+	unsigned long addmask, submask, trapmask, mask, cmask = 0;
 	unsigned branch;
 	Method handler;
 	unsigned char loopid, looplevel;
@@ -1481,32 +1698,41 @@ int ScriptImage::compile(istream *str, char *name, const char *scrname)
 	bool ignore;
 	bool unsafe = false;
 	bool access = true;
+	char temp[32];
+
+	buffer = (char *)malloc(512);
+	bufsize = 512;
 
 	scrStream = str;
 
-	enum 
+	enum
 	{
 		cSCRIPT,
 		cDATA,
 		cMAP
 	}	cmode = cSCRIPT;
 
+	gvarname[0] = '%';
+	snprintf(gvarname + 1, 48, "%s.", name);
+	gvarlen = strlen(gvarname);
+
 compile:
 	trapflag = false;
 	count = number = 0;
 	last = NULL;
 	bool req = false;
 	addmask = submask = trapmask = 0;
-	branch = 0;
 	handler = NULL;
 	loopid = looplevel = 0;
 	bool then = false;
+	events = NULL;
 
 	key = keyindex(name);
 	script = (Name *)alloc(sizeof(Name));
 	memset(script, 0, sizeof(Name));
 	script->name = alloc(name);
 	script->mask = 0;
+	script->events = NULL;
 	script->next = index[key];
 	script->mode = Name::ORIGINAL;
 	script->access = access;
@@ -1524,11 +1750,19 @@ compile:
 	for(;;)
 	{
 		quote = false;
+		paren = 0;
 		if(!then)
 		{
 			cmask = 0;
 			errflag = syncflag;
-			scrStream->getline(buffer + offset, sizeof(buffer) - 1 - offset);
+
+			if(offset > bufsize - 160)
+			{
+				bufsize += 512;
+				buffer = (char *)realloc(buffer, bufsize);
+			}
+
+			scrStream->getline(buffer + offset, bufsize - 1 - offset);
 			if(scrStream->eof())
 				break;
 
@@ -1552,6 +1786,16 @@ compile:
 		++number;
 		while(NULL != (token = getToken()))
 		{
+			if(*token == '@')
+			{
+				esave = (Name::Event *)alloc(sizeof(Name::Event));
+				esave->name = alloc(token + 1);
+				esave->line = NULL;
+				esave->next = events;
+				events = esave;
+				continue;
+			}
+
 			if(!strncmp(token, "::", 2))
 			{
 				name = token + 2;
@@ -1565,6 +1809,9 @@ compile:
 				token = "::";
 				break;
 			}
+
+			if(cmode != cSCRIPT)
+				break;
 			
 			if(*token == '^')
 			{
@@ -1575,14 +1822,7 @@ compile:
 				}
 			}
 			if(*token != '^' && *token != '+' && *token != '-' && *token != '?')
-			{
-				if(*token >= '0' && *token <= '9')
-				{
-					branch |= 1 << (*token - '0');
-				}
-				else
-					break;
-			}
+				break;
 
 			if(*token == '^')
 				mask = cmds->getTrapMask(token + 1);
@@ -1741,6 +1981,17 @@ compile:
 			continue;
 		}
 
+		if(!stricmp(token, ".export"))
+		{
+			access = true;
+			token = getToken();
+			gvarname[0] = '%';
+			snprintf(gvarname + 1, 48, "%s.", token);
+			gvarlen = strlen(gvarname);
+			basename = alloc(token);
+			continue;
+		}
+
 		if(!stricmp(token, ".public"))
 		{
 			access = true;
@@ -1759,14 +2010,39 @@ compile:
 			continue;
 		}
 
+		if(!strnicmp(token, ".initial", 8))
+		{
+			token += 8;
+			if(*token =='.')
+				++token;
+			if(!*token)
+				token = "0";
+			vsize = atoi(token);
+			strcpy(namebuf, script->name);
+			prefix = strchr(namebuf, ':');
+			if(!prefix)
+				prefix = namebuf + strlen(namebuf);			
+			*(prefix++) = '.';
+			while(NULL != (token = getToken()))
+			{
+				strcpy(prefix, token);
+				token = getToken();
+				if(!token)
+					break;
+				initial(namebuf, token, vsize);
+			}
+			continue;
+		}							
+
 		if(!stricmp(token, ".define"))
 		{
 			while(NULL != (token = getToken()))
 			{
-				if(!cmds->getLast(token))
-					cmds->setValue(token, getToken());
-				else
-					getToken();
+				snprintf(namebuf, sizeof(namebuf), "%s", token);
+				token = getToken();
+				if(!getLast(namebuf))
+					setValue(namebuf, token);
+
 			}
 			continue;
 		}
@@ -1782,7 +2058,12 @@ retoken:
 		if(!token)
 			continue;
 
-
+		if(*token == '%')
+		{
+			pretoken = token;
+			token = "number";
+		}
+		else
 		switch(cmode)
 		{
 		case cSCRIPT:
@@ -1800,7 +2081,6 @@ retoken:
 			pretoken = NULL;
 		}
 
-		
 		if(!stricmp(token, "optional") || !stricmp(token, "unsafe"))
 		{
 			ignore = true;
@@ -1846,19 +2126,61 @@ retoken:
 		command = alloc(token);
 	
 		argc = 0;
-		while(argc < 32 && NULL != (token = getToken(&pretoken)))
+		while(argc < maxargs && NULL != (token = getToken(&pretoken)))
 		{
 			if(token[0] == '$' && isalpha(token[1]))
 			{
-				token = (char *)getDefined(++token);
+				if(!stricmp(token, "$script.name"))
+					token = alloc((char *)basename);
+				else if(!stricmp(token, "$script.file"))
+					token = alloc((char *)name);
+				else if(!stricmp(token, "$script.line"))
+				{
+					sprintf(temp, "%d", number);
+					token = alloc(temp);
+				}
+				else
+					token = (char *)getDefined(++token);
 				if(!token)
 					token = "";
 			}
+			else if(token[0] == 0x01)
+				token = alloc(++token);
+			else if(token[0] == '.' && isalpha(token[1]))
+			{
+				cp = token + strlen(token) - 1;
+				if(*cp != '.')
+				{
+					gvarname[0] = '%';
+					snprintf(gvarname + gvarlen, sizeof(gvarname) - gvarlen, "%s", token + 1);
+					token = alloc(gvarname);
+				}
+				else
+					token = alloc(token);
+			}
+			else if(!strnicmp(token, "&.", 2) || !strnicmp(token, ">.", 2))
+			{
+				gvarname[0] = '&';
+                                snprintf(gvarname + gvarlen,sizeof(gvarname) - gvarlen, "%s", token + 2);
+                                token = alloc(gvarname);
+			}
+			else if(!strnicmp(token, "%.", 2))
+			{
+				gvarname[0] = '%';
+				snprintf(gvarname + gvarlen, sizeof(gvarname) - gvarlen, "%s", token + 2);
+				token = alloc(gvarname);
+			}
+			else if(*token == '>' && isalnum(token[1]))
+			{
+				token = alloc(token);
+				*token = '&';
+			}
 			else
 				token = alloc(token);
-			
+
+
 			args[argc++] = token;
-			if(!stricmp(token, "then") && !stricmp(command, "if"))
+			if(!stricmp(token, "then") && handler == &ScriptInterp::scrIf)
 			{
 				--bp;
 				*bp = ' ';
@@ -1887,7 +2209,7 @@ retoken:
 
 		addmask = submask = 0;
 
-		if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
+		if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !strnicmp(command, "do.", 3) || !stricmp(command, "foreach") || !stricmp(command, "fordata"))
 		{
 			if(!looplevel)
 				++loopid;
@@ -1895,7 +2217,7 @@ retoken:
 			line->loop = loopid * 256 + looplevel;
 		}
 
-		if(!stricmp(command, "loop"))
+		if(!stricmp(command, "loop") || !strnicmp(command, "loop.", 5))
 		{
 			line->loop = loopid * 256 + looplevel;
 			if(!looplevel)
@@ -1921,6 +2243,15 @@ retoken:
 		if(!script->first)
 			script->first = line;
 
+		while(events)
+		{
+			esave = events->next;
+			events->line = line;
+			events->next = script->events;
+			script->events = events;
+			events = esave;
+		}
+
 		if(trapmask && !last)
 		{
 			for(i = 0; i < TRAP_BITS; ++i)
@@ -1932,23 +2263,11 @@ retoken:
 				}
 			}
 		}
-		if(branch)
-		{
-			for(i = 0; i < 10; ++i)
-			{
-				if((1l << i) & branch)
-				{
-					if(!script->skip[i])
-						script->skip[i] = line;
-				}
-			}
-		}
 
 		if(last)
 			last->next = line;
 			
 		last = line;
-		branch = 0;
 	}
 	line = script->first;
 	if(!script->mask)
@@ -1970,6 +2289,7 @@ retoken:
 	}
 	scrSource.close();
 	scrSource.clear();
+	free(buffer);
 	return total;
 }
 
